function [x,f,Q] = ckfilternew(niter,y,F,G,W,mu0,state,mu_i,sigma_i)
%%% Update volatilities using Forward Filtering Backward Sampling %%%
%%% y_t = F*x_t + mu_i(ind_t) + sigma_i(ind_t)
%%% x_(t+1) = G*x_t + W*epsilon_t, where epsilon_t~N(0,1)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Input arguments
%%% niter:   number of iterations,
%%% y:       log(y^2+10^(-c)),
%%% F:       F=1,
%%% G:       persistence parameter,
%%% W:       volatility of log-volatility,
%%% mu0:     long-run mean of volatility,
%%% state:   indicator variable,
%%% mu_i:    mean of mixing component,
%%% sigma_i: st.d of mixing component.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

n=length(y);
x=zeros(niter,n);
ind=state;
gamma1=mu0*(1-G);C0=W^2./(1-G^2);
for i=1:niter
    %%% Forward Filtering
    a=zeros(1,n);R=zeros(1,n);f=zeros(1,n);Q=zeros(1,n);m=zeros(1,n);C=zeros(1,n);
    A=zeros(1,n);e=zeros(1,n);
    a(1)=G*mu0+gamma1;
    R(1)=G^2*C0+W^2;
    f(1)=F*a(1)+mu_i(ind(1));
    Q(1)=F^2*R(1)+sigma_i(ind(1))^2;
    A(1)=R(1)*F/Q(1);
    e(1)=y(1)-f(1);
    m(1)=a(1)+A(1)*e(1);
    C(1)=R(1)-A(1)^2*Q(1);
    for t=2:n
        a(t)=G*m(t-1)+gamma1;
        R(t)=G^2*C(t-1)+W^2;
        f(t)=F*a(t)+mu_i(ind(t));
        Q(t)=F^2*R(t)+sigma_i(ind(t))^2;
        A(t)=R(t)*F/Q(t);
        e(t)=y(t)-f(t);
        m(t)=a(t)+A(t)*e(t);
        C(t)=R(t)-A(t)^2*Q(t);
    end
    %%% Backward Smoothing
    x(i,end)=normrnd(m(end),sqrt(C(end)));
    for t=(n-1):-1:1
        
        var1=(G^2./W^2+1./C(t))^(-1);
        mean1=(G*x(i,t+1)/W^2+m(t)/C(t))*var1;
        x(i,t)=normrnd(mean1,sqrt(var1));
    end
 
end




